#!/usr/bin/env python
#
# winfreed - download a selection of Free Software for MS Windows.
#
# Copyright (C) 2011  Antonio Ospite <ospite@studenti.unina.it>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import sys
import glob
import json
import urllib2
from progressbar import Bar, ETA, FileTransferSpeed, Percentage, ProgressBar

# TODO make OUTPUT_DIR and LANGCODE configurable from command line
OUTPUT_DIR = 'downloads'
LANGCODE = 'en-US'

# TODO PKG_DIR in $(datadir) or something like that for python projects
PKG_DIR = 'pkgs'
CHUNK_SIZE = 8192


def get_pkg(json_file):
    with open(json_file, mode='r') as f:
        pkg = json.load(f)
        basename = os.path.basename(json_file)
        package_name = os.path.splitext(basename)[0]
        pkg['package_name'] = package_name
        f.close()
        return pkg

    return None


def process_all(path, cb):
    listing = glob.glob(os.path.join(path, '*.json'))
    for json_file in listing:
        pkg = get_pkg(json_file)
        if not pkg:
            sys.stderr.write("Error: cannot get a pkg for: %s\n" % json_file)
            continue
        cb(pkg)


def show(pkg):
    print 'Package:  ', pkg['package_name']
    print 'Program:  ', pkg['name']
    print 'Homepage: ', pkg['homepage']
    print


def download(pkg):
    # the "%s" in URLs are meant to be replaced with LANGCODE
    try:
        url = pkg['URL'] % LANGCODE
    except:
        url = pkg['URL']
        pass

    response = urllib2.urlopen(url)

    filename = ""
    if 'Content-Disposition' in  response.info():
        # Use the filename the server tells us if any,
        # re pattern from http://stackoverflow.com/questions/8035900
        content_disposition = response.info().getheader('Content-Disposition').strip()
        filename = re.findall("filename=(\S+)", content_disposition)[0]

    if filename == "":
        filename = urllib2.unquote(os.path.basename(response.geturl()))

    if filename == "":
        sys.stderr.write("Debug (%s): filename: %s url: %s\n" %(pkg['package_name'], filename, response.geturl()))
        return

    destfile = os.path.join(OUTPUT_DIR, filename)
    if os.path.exists(destfile):
        sys.stderr.write("Warning (%s): %s exists!\n" % (pkg['package_name'], destfile))
        return

    outfile = open(destfile, mode='w')

    total_size = response.info().getheader('Content-Length').strip()
    total_size = int(total_size)

    widgets = [pkg['name'], ' ', Percentage(), ' ', Bar(marker='=', left='[', right=']'),
               ' ', ETA(), ' ', FileTransferSpeed()]
    pbar = ProgressBar(widgets=widgets, maxval=total_size).start()

    bytes_so_far = 0
    while 1:
        chunk = response.read(CHUNK_SIZE)
        if not chunk:
            break

        bytes_so_far += len(chunk)
        outfile.write(chunk)
        pbar.update(bytes_so_far)
    pbar.finish()


def show_all():
    process_all(PKG_DIR, show)


def download_all():
    if os.path.exists(OUTPUT_DIR) == False:
        os.mkdir(OUTPUT_DIR, 0755)

    process_all(PKG_DIR, download)


def usage():
    usage = "winfreed - download a selection of Free Software for MS Windows.\n\n"
    usage += "usage: %s <COMMAND>\n\n" % sys.argv[0]
    usage += "COMMANDS:\n"
    usage += "\tshow        Show info about all the available packages\n"
    usage += "\tdownload    Download all the packages\n"
    print usage

if __name__ == "__main__":

    if len(sys.argv) < 2:
        usage()
        sys.exit(1)

    if sys.argv[1] == 'download':
        download_all()
    elif sys.argv[1] == 'show':
        show_all()
    else:
        usage()
        sys.exit(1)

    sys.exit(0)
